@@ -31,6 +31,7 @@ authorization from the X Consortium and the XFree86 Project.
 #include <config.h>
 #endif
 #include "Xlibint.h"
+#include "reallocarray.h"
 #include <limits.h>
 
 #if defined(XF86BIGFONT)
@@ -105,8 +106,8 @@ XFontStruct *XLoadQueryFont(
       return font_result;
     LockDisplay(dpy);
     GetReq(OpenFont, req);
-    seq = dpy->request;
-    nbytes = req->nbytes  = name ? strlen(name) : 0;
+    seq = dpy->request; /* Can't use extended sequence number here */
+    nbytes = req->nbytes = (CARD16) (name ? strlen(name) : 0);
     req->fid = fid = XAllocID(dpy);
     req->length += (nbytes+3)>>2;
     Data (dpy, name, nbytes);
@@ -166,13 +167,13 @@ XFreeFont(
 #ifdef USE_XF86BIGFONT
 	_XF86BigfontFreeFontMetrics(fs);
 #else
-	Xfree ((char *) fs->per_char);
+	Xfree (fs->per_char);
 #endif
     }
     _XFreeExtData(fs->ext_data);
-    if (fs->properties)
-	Xfree ((char *) fs->properties);
-    Xfree ((char *) fs);
+
+    Xfree (fs->properties);
+    Xfree (fs);
     return 1;
 }
 
@@ -245,11 +246,11 @@ _XQueryFont (
 	    /* nFontProps is a CARD16 */
 	    nbytes = reply.nFontProps * SIZEOF(xFontProp);
 	    if ((nbytes >> 2) <= reply_left) {
-		size_t pbytes = reply.nFontProps * sizeof(XFontProp);
-		fs->properties = Xmalloc (pbytes);
+		fs->properties = Xmallocarray (reply.nFontProps,
+                                               sizeof(XFontProp));
 	    }
 	    if (! fs->properties) {
-		Xfree((char *) fs);
+		Xfree(fs);
 		_XEatDataWords(dpy, reply_left);
 		return (XFontStruct *)NULL;
 	    }
@@ -266,13 +267,13 @@ _XQueryFont (
 	if (reply.nCharInfos < (INT_MAX / sizeof(XCharStruct))) {
 	    nbytes = reply.nCharInfos * SIZEOF(xCharInfo);
 	    if ((nbytes >> 2) <= reply_left) {
-		size_t cibytes = reply.nCharInfos * sizeof(XCharStruct);
-		fs->per_char = Xmalloc (cibytes);
+		fs->per_char = Xmallocarray (reply.nCharInfos,
+                                             sizeof(XCharStruct));
 	    }
 	}
 	if (! fs->per_char) {
-	    if (fs->properties) Xfree((char *) fs->properties);
-	    Xfree((char *) fs);
+	    Xfree(fs->properties);
+	    Xfree(fs);
 	    _XEatDataWords(dpy, reply_left);
 	    return (XFontStruct *)NULL;
 	}
@@ -489,11 +490,11 @@ _XF86BigfontQueryFont (
 	/* nFontProps is a CARD16 */
 	nbytes = reply.nFontProps * SIZEOF(xFontProp);
 	if ((nbytes >> 2) <= reply_left) {
-	    size_t pbytes = reply.nFontProps * sizeof(XFontProp);
-	    fs->properties = Xmalloc (pbytes);
+	    fs->properties = Xmallocarray (reply.nFontProps,
+                                           sizeof(XFontProp));
 	}
 	if (! fs->properties) {
-	    Xfree((char *) fs);
+	    Xfree(fs);
 	    _XEatDataWords(dpy, reply_left);
 	    return (XFontStruct *)NULL;
 	}
@@ -507,8 +508,8 @@ _XF86BigfontQueryFont (
        any real font needs, so the combined total doesn't overflow either */
     if (reply.nUniqCharInfos > ((ULONG_MAX / 2) / SIZEOF(xCharInfo)) ||
 	reply.nCharInfos > ((ULONG_MAX / 2) / sizeof(CARD16))) {
-	Xfree((char *) fs->properties);
-	Xfree((char *) fs);
+	Xfree(fs->properties);
+	Xfree(fs);
 	_XEatDataWords(dpy, reply_left);
 	return (XFontStruct *)NULL;
     }
@@ -524,15 +525,16 @@ _XF86BigfontQueryFont (
 	             + (reply.nCharInfos+1)/2 * 2 * sizeof(CARD16);
 	    pUniqCI = Xmalloc (nbytes);
 	    if (!pUniqCI) {
-		if (fs->properties) Xfree((char *) fs->properties);
-		Xfree((char *) fs);
+		Xfree(fs->properties);
+		Xfree(fs);
 		_XEatDataWords(dpy, reply_left);
 		return (XFontStruct *)NULL;
 	    }
-	    if (! (fs->per_char = Xmalloc (reply.nCharInfos * sizeof(XCharStruct)))) {
-		Xfree((char *) pUniqCI);
-		if (fs->properties) Xfree((char *) fs->properties);
-		Xfree((char *) fs);
+	    if (! (fs->per_char = Xmallocarray (reply.nCharInfos,
+                                                sizeof(XCharStruct)))) {
+		Xfree(pUniqCI);
+		Xfree(fs->properties);
+		Xfree(fs);
 		_XEatDataWords(dpy, reply_left);
 		return (XFontStruct *)NULL;
 	    }
@@ -541,15 +543,15 @@ _XF86BigfontQueryFont (
 	    for (i = 0; i < reply.nCharInfos; i++) {
 		if (pIndex2UniqIndex[i] >= reply.nUniqCharInfos) {
 		    fprintf(stderr, "_XF86BigfontQueryFont: server returned wrong data\n");
-		    Xfree((char *) pUniqCI);
-		    if (fs->properties) Xfree((char *) fs->properties);
-		    Xfree((char *) fs);
+		    Xfree(pUniqCI);
+		    Xfree(fs->properties);
+		    Xfree(fs);
 		    return (XFontStruct *)NULL;
 		}
 		/* XXX the next statement won't work if short isn't 16 bits */
 		fs->per_char[i] = * (XCharStruct *) &pUniqCI[pIndex2UniqIndex[i]];
 	    }
-	    Xfree((char *) pUniqCI);
+	    Xfree(pUniqCI);
 	} else {
 #ifdef HAS_SHM
 	    XExtData *pData;
@@ -558,8 +560,8 @@ _XF86BigfontQueryFont (
 
 	    pData = Xmalloc(sizeof(XExtData));
 	    if (!pData) {
-		if (fs->properties) Xfree((char *) fs->properties);
-		Xfree((char *) fs);
+		Xfree(fs->properties);
+		Xfree(fs);
 		return (XFontStruct *)NULL;
 	    }
 
@@ -577,9 +579,9 @@ _XF86BigfontQueryFont (
 	    if ((addr = shmat(reply.shmid, NULL, SHM_RDONLY)) == (char *)-1) {
 		if (extcodes->serverCapabilities & CAP_VerifiedLocal)
 		    fprintf(stderr, "_XF86BigfontQueryFont: could not attach shm segment\n");
-	        Xfree((char *) pData);
-	        if (fs->properties) Xfree((char *) fs->properties);
-	        Xfree((char *) fs);
+	        Xfree(pData);
+	        Xfree(fs->properties);
+	        Xfree(fs);
 		/* Stop requesting shared memory transport from now on. */
 		extcodes->serverCapabilities &= ~ XF86Bigfont_CAP_LocalShm;
 	        return (XFontStruct *)NULL;
@@ -588,13 +590,13 @@ _XF86BigfontQueryFont (
 	    if (!(extcodes->serverCapabilities & CAP_VerifiedLocal)) {
 		struct shmid_ds buf;
 		if (!(shmctl(reply.shmid, IPC_STAT, &buf) >= 0
-		      && reply.nCharInfos < (LONG_MAX / sizeof(XCharStruct))
+		      && reply.nCharInfos < (INT_MAX / sizeof(XCharStruct))
 		      && buf.shm_segsz >= reply.shmsegoffset + reply.nCharInfos * sizeof(XCharStruct) + sizeof(CARD32)
 		      && *(CARD32 *)(addr + reply.shmsegoffset + reply.nCharInfos * sizeof(XCharStruct)) == extcodes->serverSignature)) {
 		    shmdt(addr);
-		    Xfree((char *) pData);
-		    if (fs->properties) Xfree((char *) fs->properties);
-		    Xfree((char *) fs);
+		    Xfree(pData);
+		    Xfree(fs->properties);
+		    Xfree(fs);
 		    /* Stop requesting shared memory transport from now on. */
 		    extcodes->serverCapabilities &= ~ XF86Bigfont_CAP_LocalShm;
 		    return (XFontStruct *)NULL;
@@ -611,8 +613,8 @@ _XF86BigfontQueryFont (
 	    fs->per_char = (XCharStruct *) (addr + reply.shmsegoffset);
 #else
 	    fprintf(stderr, "_XF86BigfontQueryFont: try recompiling libX11 with HasShm, Xserver has shm support\n");
-	    if (fs->properties) Xfree((char *) fs->properties);
-	    Xfree((char *) fs);
+	    if (fs->properties) Xfree(fs->properties);
+	    Xfree(fs);
 	    /* Stop requesting shared memory transport from now on. */
 	    extcodes->serverCapabilities &= ~ XF86Bigfont_CAP_LocalShm;
 	    return (XFontStruct *)NULL;
@@ -638,9 +640,9 @@ _XF86BigfontFreeFontMetrics (XFontStruct *fs)
 				      XF86BigfontNumber)))
 	shmdt ((char *) pData->private_data);
     else
-	Xfree ((char *) fs->per_char);
+	Xfree (fs->per_char);
 #else
-    Xfree ((char *) fs->per_char);
+    Xfree (fs->per_char);
 #endif
 }
 
@@ -660,7 +662,7 @@ int _XF86LoadQueryLocaleFont(
 
     if (!name)
 	return 0;
-    l = strlen(name);
+    l = (int) strlen(name);
     if (l < 2 || name[l - 1] != '*' || name[l - 2] != '-')
 	return 0;
     charset = NULL;
@@ -677,7 +679,7 @@ int _XF86LoadQueryLocaleFont(
 	return 0;
     if (_XlcNCompareISOLatin1(name + l - 2 - (p - charset), charset, p - charset))
 	return 0;
-    if (strlen(p + 1) + l - 1 >= sizeof(buf) - 1)
+    if (strlen(p + 1) + (size_t) l - 1 >= sizeof(buf) - 1)
 	return 0;
     strcpy(buf, name);
     strcpy(buf + l - 1, p + 1);
@@ -693,14 +695,14 @@ int _XF86LoadQueryLocaleFont(
 #ifdef USE_XF86BIGFONT
 	    _XF86BigfontFreeFontMetrics(fs);
 #else
-	    Xfree ((char *) fs->per_char);
+	    Xfree (fs->per_char);
 #endif
 	}
 	_XFreeExtData(fs->ext_data);
-	if (fs->properties)
-	    Xfree ((char *) fs->properties);
+
+	Xfree (fs->properties);
 	*fidp = fs->fid;
-	Xfree ((char *) fs);
+	Xfree (fs);
     } else {
 	XFreeFont(dpy, fs);
     }
