@@ -48,11 +48,6 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/keysymdef.h>
 #include <stdio.h>
 
-#ifdef USE_OWN_COMPOSE
-#include "imComp.h"
-
-#endif
-
 #include "Xresource.h"
 #include "Key.h"
 
@@ -239,7 +234,7 @@ XRefreshKeyboardMapping(register XMappingEvent *event)
 	 */
 	LockDisplay(event->display);
 	if (event->display->keysyms) {
-	     Xfree ((char *)event->display->keysyms);
+	     Xfree (event->display->keysyms);
 	     event->display->keysyms = NULL;
 	}
 	UnlockDisplay(event->display);
@@ -276,12 +271,13 @@ _XKeyInitialize(
 	if (! keysyms) return 0;
 
 	LockDisplay(dpy);
-	if (dpy->keysyms)
-	    Xfree ((char *)dpy->keysyms);
+
+	Xfree (dpy->keysyms);
 	dpy->keysyms = keysyms;
 	dpy->keysyms_per_keycode = per;
 	if (dpy->modifiermap)
 	    ResetModMap(dpy);
+
 	UnlockDisplay(dpy);
     }
     if (!dpy->modifiermap)
@@ -470,6 +466,8 @@ UCSConvertCase( register unsigned code,
             *upper = 0x0178;
         else if (code == 0x00b5)      /* micro sign */
             *upper = 0x039c;
+        else if (code == 0x00df)      /* ssharp */
+            *upper = 0x1e9e;
 	return;
     }
 
@@ -599,6 +597,8 @@ UCSConvertCase( register unsigned code,
         }
         else if (code == 0x1e9b)
             *upper = 0x1e60;
+        else if (code == 0x1e9e)
+            *lower = 0x00df;    /* ssharp */
     }
 
     /* Greek Extended, U+1F00 to U+1FFF */
@@ -741,8 +741,9 @@ XConvertCase(
 	    *upper -= (XK_Greek_alphaaccent - XK_Greek_ALPHAaccent);
 	else if (sym >= XK_Greek_ALPHA && sym <= XK_Greek_OMEGA)
 	    *lower += (XK_Greek_alpha - XK_Greek_ALPHA);
-	else if (sym >= XK_Greek_alpha && sym <= XK_Greek_omega &&
-		 sym != XK_Greek_finalsmallsigma)
+	else if (sym == XK_Greek_finalsmallsigma)
+	    *upper = XK_Greek_SIGMA;
+	else if (sym >= XK_Greek_alpha && sym <= XK_Greek_omega)
 	    *upper -= (XK_Greek_alpha - XK_Greek_ALPHA);
         break;
     case 0x13: /* Latin 9 */
@@ -836,7 +837,7 @@ _XTranslateKeySym(
 	if (((modifiers & AllMods) == p->state) && (symbol == p->key)) {
 	    length = p->len;
 	    if (length > nbytes) length = nbytes;
-	    memcpy (buffer, p->string, length);
+	    memcpy (buffer, p->string, (size_t) length);
 	    return length;
 	}
     }
@@ -891,73 +892,6 @@ XLookupString (
 		  &modifiers, &symbol))
 	return 0;
 
-#ifdef USE_OWN_COMPOSE
-    if ( status ) {
-	static int been_here= 0;
-	if ( !been_here ) {
-	    XimCompInitTables();
-	    been_here = 1;
-	}
-	if ( !XimCompLegalStatus(status) ) {
-	    status->compose_ptr = NULL;
-	    status->chars_matched = 0;
-	}
-	if ( ((status->chars_matched>0)&&(status->compose_ptr!=NULL)) ||
-		XimCompIsComposeKey(symbol,event->keycode,status) ) {
-	    XimCompRtrn rtrn;
-	    switch (XimCompProcessSym(status,symbol,&rtrn)) {
-		case XIM_COMP_IGNORE:
-		    break;
-		case XIM_COMP_IN_PROGRESS:
-		    if ( keysym!=NULL )
-			*keysym = NoSymbol;
-		    return 0;
-		case XIM_COMP_FAIL:
-		{
-		    int n = 0, len= 0;
-		    for (n=len=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    if ( keysym!=NULL ) {
-			if ( n==1 )	*keysym = rtrn.sym[0];
-			else		*keysym = NoSymbol;
-		    }
-		    return len;
-		}
-		case XIM_COMP_SUCCEED:
-		{
-		    int len,n = 0;
-
-		    symbol = rtrn.matchSym;
-		    if ( keysym!=NULL )	*keysym = symbol;
-		    if ( rtrn.str[0]!='\0' ) {
-			strncpy(buffer,rtrn.str,nbytes-1);
-			buffer[nbytes-1]= '\0';
-			len = strlen(buffer);
-		    }
-		    else {
-			len = _XTranslateKeySym(event->display,symbol,
-							event->state,
-							buffer,nbytes);
-		    }
-		    for (n=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    return len;
-		}
-	    }
-	}
-    }
-#endif
-
     if (keysym)
 	*keysym = symbol;
     /* arguable whether to use (event->state & ~modifiers) here */
@@ -974,8 +908,8 @@ _XFreeKeyBindings(
     for (p = dpy->key_bindings; p; p = np) {
 	np = p->next;
 	Xfree(p->string);
-	Xfree((char *)p->modifiers);
-	Xfree((char *)p);
+	Xfree(p->modifiers);
+	Xfree(p);
     }
 }
 
@@ -1001,9 +935,9 @@ XRebindKeysym (
 	((! (p->string = Xmalloc(nbytes))) && (nbytes > 0)) ||
 	((! (p->modifiers = Xmalloc(nb))) && (nb > 0))) {
 	if (p) {
-	    if (p->string) Xfree(p->string);
-	    if (p->modifiers) Xfree((char *) p->modifiers);
-	    Xfree((char *) p);
+	    Xfree(p->string);
+	    Xfree(p->modifiers);
+	    Xfree(p);
 	}
 	UnlockDisplay(dpy);
 	return 0;
@@ -1012,9 +946,9 @@ XRebindKeysym (
     dpy->key_bindings = p;
     dpy->free_funcs->key_bindings = _XFreeKeyBindings;
     p->next = tmp;	/* chain onto list */
-    memcpy (p->string, str, nbytes);
+    memcpy (p->string, str, (size_t) nbytes);
     p->len = nbytes;
-    memcpy ((char *) p->modifiers, (char *) mlist, nb);
+    memcpy ((char *) p->modifiers, (char *) mlist, (size_t) nb);
     p->key = keysym;
     p->mlen = nm;
     ComputeMaskFromKeytrans(dpy, p);
